-- -------------------------------------------------------------------------- --
--                            DrainSouler by kunda                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- Shows a small movable frame with all data you need to know for Drain Soul. --
--                                                                            --
-- Framesize: 80x20px                                                         --
-- Elements:                                                                  --
--  - HP (in percent)                                                         --
--  - Icon (visible if target HP is below 25%)                                --
--  - TickTimer (a progress bar that indicates when the next Drain Soul Tick  --
--              is expected)                                                  --
--  - Tick (the number of Damage Ticks since caststart)                       --
--  - DMG (Drain Soul damage from last Tick)                                  --
-- Optional Configuration:                                                    --
--  - Look at top of DrainSouler.lua.                                         --
--                                                                            --
-- TRICKY TICK:                                                               --
--  Start if HP is below 25%!                                                 --
--  Stop channeling Drain Soul after 4 Ticks (unless you need shards)!        --
--  Drain Soul does never deal 5 times damage! Stop and cast it again.        --
--  In most situations you need to reapply DoTs earlier (second or third      --
--  Tick). Reapply them after a Drain Soul Tick!                              --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
--                                    STOP                                    --
--                 You must be a Warlock to enter this code.                  --
--                  If you are not a Warlock please leave.                    --
        if select(2, UnitClass("player")) ~= "WARLOCK" then return end          
-- -------------------------------------------------------------------------- --

-- Optional Configuration START ----------------------------------------------------------------------------------------
local minHP = 0            -- example: 'local minHP = 87000' -> if maximum target HP is below 87000 the frame is NOT shown
local minHPP = 100         -- example: 'local minHPP = 50' -> if target HP is below 50% the frame is shown
local TickSound = nil      -- example: 'local TickSound = "RaidWarning"' -> plays a sound when DrainSoul dealt damage
-- Optional Configuration END ------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
DrainSouler_Options = {}

local DrainSouler = CreateFrame("Frame")

local _G               = _G
local GetTime          = _G.GetTime
local UnitHealth       = _G.UnitHealth
local UnitHealthMax    = _G.UnitHealthMax
local UnitChannelInfo  = _G.UnitChannelInfo
local math_floor       = _G.math.floor

local mult2 = 10^(2 or 0)
local PlayerGUID
local DrainSoulName, _, DrainSoulIcon = GetSpellInfo(1120)

local numTicks = 5
local DrainSoulCurrentDuration
local DrainSoulCurrentStart
local DrainSoulCurrentTick = 0
local DrainSoulIsActive
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function CheckHealth()
	local maxHealth = UnitHealthMax("target")
	if maxHealth < minHP then
		DrainSouler_Frame:Hide()
		return
	end
	local health = UnitHealth("target") / maxHealth * 100
	if health > 0 and health <= minHPP then
		DrainSouler_Frame:Show()
		DrainSouler_Text_HP:SetText( math_floor(health * mult2 + 0.5) / mult2.."%" )
		if health < 25 then
			DrainSouler_Button_Icon:Show()
		else
			DrainSouler_Button_Icon:Hide()
		end
	else
		DrainSouler_Frame:Hide()
		DrainSouler_Text_HP:SetText("")
		DrainSouler_Text_DMG:SetText("")
		DrainSouler_Text_Tick:SetText("")
	end
	if not DrainSoulIsActive then
		DrainSouler_Button_TickTimer:Hide()
	end
end

local function TickTimer()
	local width = 60 - (( (GetTime()*1000) - DrainSoulCurrentStart)*60) / (DrainSoulCurrentDuration/ (numTicks-DrainSoulCurrentTick) )
	if width < 0.01 then
		width = 0.01
	end
	DrainSouler_Texture_TickTimer:SetWidth(width)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function DrainSouler:InitFrame()
	local DrainSouler = CreateFrame("Frame", "DrainSouler_Frame")
	DrainSouler:SetClampedToScreen(true)
	DrainSouler:SetMovable(true)
	DrainSouler:EnableMouse(true)
	DrainSouler:SetPoint("CENTER", UIParent, 0, 200)
	DrainSouler:SetWidth(1)
	DrainSouler:SetHeight(1)
	DrainSouler:SetScale(1)
	DrainSouler:Hide()

	local function OnMouseDown()
		DrainSouler_Frame:StartMoving()
	end
	local function OnMouseUp()
		DrainSouler_Frame:StopMovingOrSizing()
		DrainSouler_Options.posX = DrainSouler_Frame:GetLeft()
		DrainSouler_Options.posY = DrainSouler_Frame:GetTop()
	end

	-- Why a button for each? In the beginning I want to make a show/hide option for every element,
	-- but I came to the conclusion that it's better and easier without such option(s).
	-- Maybe I change this someday...but I am now too lazy to remove the buttons ;)
	local icon = CreateFrame("Button", "DrainSouler_Button_Icon", DrainSouler_Frame)
	icon:SetFrameLevel(DrainSouler_Frame:GetFrameLevel()+1)
	icon:SetWidth(20)
	icon:SetHeight(20)
	icon:ClearAllPoints()
	icon:SetPoint("TOPLEFT", "DrainSouler_Frame", "TOPLEFT", 0, 0)
	icon:SetScript("OnMouseDown", OnMouseDown)
	icon:SetScript("OnMouseUp", OnMouseUp)
	icon:SetScript("OnHide", OnMouseUp)
	local background = icon:CreateTexture(nil, "BACKGROUND")
	background:SetAllPoints(icon)
	background:SetTexture(0, 0, 0, 1)
	local texture = icon:CreateTexture(nil, "BACKGROUND")
	texture:SetWidth(18)
	texture:SetHeight(18)
	texture:SetPoint("TOPLEFT", "DrainSouler_Button_Icon", "TOPLEFT", 1, -1)
	texture:SetTexture(DrainSoulIcon)

	local HP = CreateFrame("Button", "DrainSouler_Button_HP", DrainSouler_Frame)
	HP:SetFrameLevel(DrainSouler_Frame:GetFrameLevel()+1)
	HP:SetWidth(60)
	HP:SetHeight(10)
	HP:ClearAllPoints()
	HP:SetPoint("TOPLEFT", "DrainSouler_Frame", "TOPLEFT", 20, 0)
	HP:SetScript("OnMouseDown", OnMouseDown)
	HP:SetScript("OnMouseUp", OnMouseUp)
	HP:SetScript("OnHide", OnMouseUp)
	local background = HP:CreateTexture(nil, "BACKGROUND")
	background:SetAllPoints(HP)
	background:SetTexture(0, 0, 0, 1)
	local text = HP:CreateFontString("DrainSouler_Text_HP", "ARTWORK", "GameFontWhiteTiny")
	text:SetAllPoints(HP)
	text:SetJustifyH("CENTER")
	text:SetJustifyV("MIDDLE")

	local DMG = CreateFrame("Button", "DrainSouler_Button_DMG", DrainSouler_Frame)
	DMG:SetFrameLevel(DrainSouler_Frame:GetFrameLevel()+1)
	DMG:SetWidth(60)
	DMG:SetHeight(10)
	DMG:ClearAllPoints()
	DMG:SetPoint("TOPLEFT", "DrainSouler_Frame", "TOPLEFT", 20, -10)
	DMG:SetScript("OnMouseDown", OnMouseDown)
	DMG:SetScript("OnMouseUp", OnMouseUp)
	DMG:SetScript("OnHide", OnMouseUp)
	local background = DMG:CreateTexture(nil, "BACKGROUND")
	background:SetAllPoints(DMG)
	background:SetTexture(0, 0, 0, 1)
	local text = DMG:CreateFontString("DrainSouler_Text_DMG", "ARTWORK", "GameFontWhiteTiny")
	text:SetAllPoints(DMG)
	text:SetJustifyH("CENTER")
	text:SetJustifyV("MIDDLE")

	local TickTimer = CreateFrame("Button", "DrainSouler_Button_TickTimer", DrainSouler_Frame)
	TickTimer:SetFrameLevel(DrainSouler_Frame:GetFrameLevel()+1)
	TickTimer:SetWidth(60)
	TickTimer:SetHeight(10)
	TickTimer:ClearAllPoints()
	TickTimer:SetPoint("TOPLEFT", "DrainSouler_Frame", "TOPLEFT", 20, -10)
	TickTimer:SetScript("OnMouseDown", OnMouseDown)
	TickTimer:SetScript("OnMouseUp", OnMouseUp)
	TickTimer:SetScript("OnHide", OnMouseUp)
	local texture = TickTimer:CreateTexture("DrainSouler_Texture_TickTimer", "ARTWORK")
	texture:SetWidth(0.01)
	texture:SetHeight(6)
	texture:SetPoint("TOPLEFT", "DrainSouler_Frame", "TOPLEFT", 20, -12)
	texture:SetTexture(0.12, 0.34, 0.45, 1)

	local Tick = CreateFrame("Button", "DrainSouler_Button_Tick", DrainSouler_Frame)
	Tick:SetFrameLevel(DrainSouler_Frame:GetFrameLevel()+2)
	Tick:SetWidth(20)
	Tick:SetHeight(20)
	Tick:ClearAllPoints()
	Tick:SetPoint("TOPLEFT", "DrainSouler_Frame", "TOPLEFT", 20, 0)
	Tick:SetScript("OnMouseDown", OnMouseDown)
	Tick:SetScript("OnMouseUp", OnMouseUp)
	Tick:SetScript("OnHide", OnMouseUp)
	local background = Tick:CreateTexture(nil, "BACKGROUND")
	background:SetAllPoints(Tick)
	background:SetTexture(1, 1, 1, 0)
	local text = Tick:CreateFontString("DrainSouler_Text_Tick", "ARTWORK", "GameFontHighlightLarge")
	text:SetAllPoints(Tick)
	text:SetJustifyH("LEFT")
	text:SetJustifyV("MIDDLE")

	if DrainSouler_Options.posX then
		DrainSouler_Frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", DrainSouler_Options.posX, DrainSouler_Options.posY)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event, ...)
	if event == "UNIT_HEALTH" then
		if arg1 == "target" then 
			CheckHealth()
		end
	elseif event == "PLAYER_TARGET_CHANGED" then
		if UnitExists("target") then
			CheckHealth()
			if DrainSouler_Frame:IsShown() then
				DrainSouler:RegisterEvent("UNIT_HEALTH")
			end
		else
			DrainSouler_Frame:Hide()
			DrainSouler:UnregisterEvent("UNIT_HEALTH")
		end
	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
		if arg1 == "player" and arg2 == DrainSoulName then
			local _, _, _, _, startTime, endTime = UnitChannelInfo("player")
			if startTime and endTime then
				DrainSoulIsActive = true
				DrainSouler:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				numTicks = 5
				DrainSoulCurrentDuration = endTime-startTime
				DrainSoulCurrentStart = startTime
				DrainSouler_Button_TickTimer:Show()
				DrainSouler_Button_TickTimer:SetScript("OnUpdate", TickTimer)
			end
		end
	elseif event == "UNIT_SPELLCAST_CHANNEL_UPDATE" then
		if arg1 == "player" and arg2 == DrainSoulName then
			local _, _, _, _, _, endTime = UnitChannelInfo("player")
			numTicks = 4
			--DrainSoulCurrentStart = GetTime()*1000
			DrainSoulCurrentDuration = endTime-DrainSoulCurrentStart
		end
	elseif event == "UNIT_SPELLCAST_CHANNEL_STOP" then
		if arg1 == "player" and arg2 == DrainSoulName then
			DrainSouler_Button_TickTimer:SetScript("OnUpdate", nil)
			DrainSoulCurrentDuration = nil
			DrainSoulCurrentTick = 0
			DrainSoulIsActive = nil
			DrainSouler_Text_Tick:SetText("")
			DrainSouler_Text_DMG:SetText("")
			DrainSouler_Button_TickTimer:Hide()
			DrainSouler:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
		if arg3 ~= PlayerGUID then return end
		if arg10 == DrainSoulName and arg2 == "SPELL_PERIODIC_DAMAGE" then
			local _, _, _, _, _, endTime = UnitChannelInfo("player")
			if endTime then
				DrainSoulCurrentStart = GetTime()*1000
				DrainSoulCurrentDuration = endTime-DrainSoulCurrentStart
			end
			DrainSoulCurrentTick = DrainSoulCurrentTick + 1
			if TickSound then PlaySound(TickSound) end
			DrainSouler_Text_Tick:SetText(DrainSoulCurrentTick)
			DrainSouler_Text_DMG:SetText(arg12)
		end
	elseif event == "PLAYER_LOGIN" then
		DrainSouler:InitFrame()
		PlayerGUID = UnitGUID("player")
		DrainSouler:UnregisterEvent("PLAYER_LOGIN")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

DrainSouler:RegisterEvent("PLAYER_LOGIN")
DrainSouler:RegisterEvent("PLAYER_TARGET_CHANGED")
DrainSouler:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
DrainSouler:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
DrainSouler:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
DrainSouler:SetScript("OnEvent", OnEvent)